MyProtocolID = $02
#include "crash82.inc"
#include "crslave.h"

.db "Talk via MBus 1.1",0

	ROM_CALL(CLEARLCD)			; clear LCD

	SI2C_PRESENT				; make sure MBus is loaded
	jr	z, Main_Init			; if so, jump to initialization routine
	ld	hl, Str_NoMBus			; if not, display error message and exit
	ROM_CALL(D_ZT_STR)
	jp	CR_KHAND
Main_Init:
	call	ClearOwnMessage			; clear own message
	ld	a, 0				; clear partner's message
	ld	(Str_PartnersMessage), a
	SI2C_ENABLE				; enable reception

Main_KeyLoop:
	halt

	call	CheckPartnerMsg			; check for message from partner

	call	GET_KEY				; check for keypresses
	or	a				; no key pressed -> new loop
	jr	z, Main_KeyLoop
	cp	$09				; "ENTER" -> send text
	jr	z, SendText
	cp	$37				; "MODE" -> exit
	jr	z, Main_Exit

	ld	b, 0				; lookup key in table and store it into a
	ld	c, a
	ld	hl, KeyTab-1
	add	hl, bc
	ld	a, (hl)
	ld	bc, (Cursor)			; goto cursor position in own message string
	ld	hl, Str_OwnMessage
	add	hl, bc
	ld	(hl), a				; put new key
	ld	a, 63-1				; already at the end of message?
	cp	c
	jr	z, Main_KeyLoop_1
	inc	c				; no, move cursor
	ld	(Cursor), bc
Main_KeyLoop_1:					; yes, don't move cursor

	call	DrawScreen			; draw screen (with new message)

	jr	Main_KeyLoop			; return to key loop

;-----------------------------------------------

Main_Exit:
	ld	de, Str_OwnMessage		; copy exit message
	ld	hl, Str_Leave
	ld	bc, 14+1
	ldir

	call	SendOwnMsg			; send (exit) message

	SI2C_DISABLE				; disable reception
	ret					; exit to CrASH

;-----------------------------------------------

SendText:
	call	SendOwnMsg			; send message

	call	ClearOwnMessage			; clear own message
	ld	(Cursor), bc			; reset cursor position (set it to 0)
	call	DrawScreen			; draw screen (without own message)

	jr	Main_KeyLoop			; return to key loop

;-----------------------------------------------

CheckPartnerMsg:
	ld	a, (I2CInputLength)		; any packet received?
	or	a
	ret	z				; no, return to key loop

	ld	a, (I2CProtID)			; check protocol ID
	cp	$FF				; ReqInfo?
	jr	z, CheckPartnerMsg_2		; yes, cancel
	cp	MyProtocolID			; Talk message?
	jr	nz, CheckPartnerMsg_1		; no, other protocol -> cancel

	ld	hl, I2CInputData		; copy message
	ld	de, Str_PartnersMessage
	ld	bc, 63
	ldir

	call	DrawScreen			; draw screen (with new partner's message)

CheckPartnerMsg_1:
	xor	a				; enable reception of further packets
	ld	(I2CInputLength), a
	ret					; return

CheckPartnerMsg_2:
	ld	a, (I2COriginAddr)		; receiver = sender of SendInfo
	and	%11111110			; real address, even if SendInfo was a general call
	ld	d, a
	ld	a, $FE				; SendInfo protocol
	ld	b, 10				; length of message
	ld	hl, Str_SendInfo		; load message
	call	MI2C				; send data
	jr	CheckPartnerMsg_1		; operation complete

;-----------------------------------------------

SendOwnMsg:
	ld	a, MyProtocolID			; send own message to all calcs
	ld	b, 63
	ld	d, 0
	ld	hl, Str_OwnMessage
	call	MI2C
	ret					; return

;-----------------------------------------------

DrawScreen:
	ROM_CALL(CLEARLCD)			; clear lcd (old partner's message)
	ld	hl, $0000			; print own message
	ld	(CURSOR_POS), hl
	ld	hl, Str_OwnMessage
	ROM_CALL(D_ZT_STR)
	ld	hl, $0004			; print partner's message
	ld	(CURSOR_POS), hl
	ld	hl, Str_PartnersMessage
	ROM_CALL(D_ZT_STR)
	ret					; return

;-----------------------------------------------

ClearOwnMessage:
	ld	hl, Str_OwnMessage		; clear own message
	ld	de, Str_OwnMessage+1
	ld	(hl), ' '
	ld	bc, 63-1
	ldir
	ret					; return

;===============================================

#define NO_RAW
I2CSpeed:
	.db $04
#include "crmaster.inc"

;===============================================

Cursor:
	.dw 0

Str_OwnMessage:
	.db "                                                               ",0   ; 63+1 characters

Str_PartnersMessage:
	.db "                                                               ",0   ; 63+1 characters

;===============================================

Str_NoMBus:
	.db "You   have    to"
	.db "install the MBus"
	.db "drivers first",0

Str_SendInfo:
	.db "Talk 1.1",0,82			; 8+1+1 characters	; TI-82 with Talk 1.1

Str_Leave:
	.db "I'm leaving...",0			; 14+1 characters

KeyTab:
	.db "         ",34,"WRMH  ? VQLG  .ZUPKFC  YTOJEB  XSNIDA         "
